@@ -39,6 +39,10 @@ try:
     from jupyterlab import __version__ as jupyterlab_version
 except:
     jupyterlab_version = 'No JupyterLab'
+try:
+    from notebook import __version__ as notebook_version
+except:
+    notebook_version = 'No notebook'
 
 import analytics
 
@@ -187,6 +191,7 @@ def log(log_event: str, params: Dict[Any, Any]=None, steps_manager: StepsManager
     python_properties = {
         'version_python': sys.version_info,
         'version_jupyterlab': jupyterlab_version,
+        'version_notebook': notebook_version,
         'version_mito': __version__,
         'package_name': package_name,
         'location': location,
@@ -252,8 +257,18 @@ def log(log_event: str, params: Dict[Any, Any]=None, steps_manager: StepsManager
                     private_params[key] = value
             elif 'graph_creation' in key:
                 # Don't log the column ids in the graph, just log the number of series graphed
-                private_params[key]['graph_creation']['x_axis_column_ids'] = len(private_params[key]['graph_creation']['x_axis_column_ids'])
-                private_params[key]['graph_creation']['y_axis_column_ids'] = len(private_params[key]['graph_creation']['y_axis_column_ids'])
+                private_params['params_x_axis_column_ids'] = len(value['x_axis_column_ids'])
+                private_params['params_y_axis_column_ids'] = len(value['y_axis_column_ids'])
+                private_params['params_color'] = True if 'color' in value.keys() else False
+            elif 'graph_styling' in key:
+                private_params['params_title'] = True if 'title' in value['title'] else False
+                private_params['params_title_visible'] = value['title']['visible']
+                private_params['params_xaxis_title'] = True if 'title' in value['xaxis'] else False
+                private_params['params_xaxis_title_visible'] = value['xaxis']['visible']
+                private_params['params_xaxis_rangeslider_visible'] = value['xaxis']['rangeslider']['visible']
+                private_params['params_yaxis_title'] = True if 'title' in value['yaxis'] else False
+                private_params['params_yaxis_title_visible'] = value['yaxis']['visible']
+                private_params['params_showlegend'] = value['showlegend']
             elif 'sheet_index' in key:
                 private_params[key] = value
                 # Make sure the steps manager exists, and the source is in bounds
@@ -309,6 +324,7 @@ def identify() -> None:
             'version_mito': __version__,
             'package_name': package_name, 
             'version_jupyterlab': jupyterlab_version,
+            'version_notebook': notebook_version,
             'operating_system': operating_system,
             'email': user_email,
             'local': local,
@@ -382,7 +398,6 @@ def log_event_processed(event: Dict[str, Any], steps_manager: StepsManagerType,
         # NOTE: we keep this as underscored with wsc for backwards compatibility with
         # our logs!
         steps_manager_properties = {
-            # NOTE: Analysis name is the UUID that mito saves the analysis under
             'wsc_analysis_name': steps_manager.analysis_name,
             # NOTE: Change this when code fixing this logic is merged in
             'wsc_data_type_in_mito': str(steps_manager.data_type_in_mito),
@@ -391,12 +406,6 @@ def log_event_processed(event: Dict[str, Any], steps_manager: StepsManagerType,
             'wsc_curr_step_type': steps_manager.curr_step.step_type,
         }
 
-        if event['type'] == 'replay_analysis_update' or event['type'] == 'save_analysis_update':
-            # We log the steps, when the user saves or replays an analysis
-            steps_manager_properties['steps_manager_steps'] = [
-                step.step_type for step in steps_manager.steps
-            ]
-
         # We also check there is an edit_error, and if there is, then we add the error logs
         if mito_error is not None:
             recent_traceback = get_recent_traceback_as_list()
