@@ -9,6 +9,7 @@ how to add a step!
 """
 
 from typing import Dict, List, Type
+from mitosheet.step_performers.concat import ConcatStepPerformer
 from mitosheet.step_performers.drop_duplicates import DropDuplicatesStepPerformer
 from mitosheet.step_performers.import_steps.excel_import import ExcelImportStepPerformer
 from mitosheet.step_performers.step_performer import StepPerformer
@@ -29,7 +30,10 @@ from mitosheet.step_performers.dataframe_steps.dataframe_delete import Dataframe
 from mitosheet.step_performers.dataframe_steps.dataframe_duplicate import DataframeDuplicateStepPerformer
 from mitosheet.step_performers.dataframe_steps.dataframe_rename import DataframeRenameStepPerformer
 from mitosheet.step_performers.bulk_old_rename.bulk_old_rename import BulkOldRenameStepPerformer
-from mitosheet.step_performers.graph.graph import GraphStepPerformer
+from mitosheet.step_performers.graph_steps.graph import GraphStepPerformer
+from mitosheet.step_performers.graph_steps.graph_delete import GraphDeleteStepPerformer
+from mitosheet.step_performers.graph_steps.graph_duplicate import GraphDuplicateStepPerformer
+from mitosheet.step_performers.graph_steps.graph_rename import GraphRenameStepPerformer
 
 
 # All steps must be listed in this variable. Note the Type annotation allows for
@@ -45,6 +49,7 @@ STEP_PERFORMERS: List[Type[StepPerformer]] = [
     ChangeColumnDtypeStepPerformer,
     ChangeColumnFormatStepPerformer,
     MergeStepPerformer,
+    ConcatStepPerformer,
     DeleteColumnStepPerformer,
     RenameColumnStepPerformer,
     SimpleImportStepPerformer,
@@ -54,7 +59,10 @@ STEP_PERFORMERS: List[Type[StepPerformer]] = [
     DataframeRenameStepPerformer,
     BulkOldRenameStepPerformer,
     DropDuplicatesStepPerformer,
-    GraphStepPerformer
+    GraphStepPerformer,
+    GraphDeleteStepPerformer,
+    GraphDuplicateStepPerformer,
+    GraphRenameStepPerformer
 ]
 
 # A helpful mapping for looking up steps based on the incoming events
